/**
 * BLOCK: ecc-block
 *
 * Registering a basic block with Gutenberg.
 * Simple block, renders and saves the same content without any interactivity.
 */

//  Import CSS.
import './style.scss';
import './editor.scss';

const { __ } = wp.i18n; // Import __() from wp.i18n
const { registerBlockType, InspectorControls } = wp.blocks; // Import registerBlockType() from wp.blocks
const { SelectControl } = wp.components;
const { Component } = wp.element;

/**
 * Register: aa Gutenberg Block.
 *
 * Registers a new block provided a unique name and an object defining its
 * behavior. Once registered, the block is made editor as an option to any
 * editor interface where blocks are implemented.
 *
 * @link https://wordpress.org/gutenberg/handbook/block-api/
 * @param  {string}   name     Block name.
 * @param  {Object}   settings Block settings.
 * @return {?WPBlock}          The block, if it has been successfully
 *                             registered; otherwise `undefined`.
 */

class mySelectPosts extends Component {
	// Method for setting the initial state.
	static getInitialState( selectedPost ) {
		return {
			posts: [],
			selectedPost: selectedPost,
			post: {},
		};
	}
	// Constructing our component. With super() we are setting everything to 'this'.
	// Now we can access the attributes with this.props.attributes
	constructor() {
		super( ...arguments );
		this.state = this.constructor.getInitialState( this.props.attributes.selectedPost );
		// Bind so we can use 'this' inside the method.
		this.getOptions = this.getOptions.bind(this);
		// Load posts.
		this.getOptions();
		this.onChangeSelectPost = this.onChangeSelectPost.bind(this);
	}

	onChangeSelectPost( value ) {
		// Find the post
		const post = this.state.posts.find( ( item ) => { return item.id == parseInt( value ) } );
		if(post){
			// Set the state
			this.setState( { selectedPost: parseInt( value ), post } );
			// Set the attributes
			this.props.setAttributes( {
				selectedPost: parseInt( value ),
				id: post.id,
				title: post.title.rendered,
			});
		}else{
			// Set the state
			this.setState( { selectedPost: null, post: null } );
			// Set the attributes
			this.props.setAttributes( {
				selectedPost: null,
				id: null,
				title: null
			});
		}
	}

	/**
	 * Loading Posts
	 */
	getOptions() {
		return wp.apiFetch( { path: '/wp/v2/ecc-converters' } ).then( posts => {
			if( posts && 0 !== this.state.selectedPost ) {
				// If we have a selected Post, find that post and add it.
				const post = posts.find( ( item ) => { return item.id == this.state.selectedPost } );
				// This is the same as { post: post, posts: posts }
				this.setState( { post, posts } );
			} else {
				this.state.posts = posts;
				this.setState({ posts });
			}
		});
	}


	render() {
		let options = [ { value: 0, label: __( 'Select a Converter' ) } ];
		let output  = __( 'Loading Posts' );
		if( this.state.posts.length > 0 ) {
			const loading = __( 'We have %d converters available. Choose one.' );
			output = loading.replace( '%d', this.state.posts.length );
			this.state.posts.forEach((post) => {
				options.push({value:post.id, label:post.title.rendered});
			});
		} else {
			output = __( 'No converters found. Please create some first.' );
		}
		if( this.state.post && this.state.post.hasOwnProperty('title') ) {
			output = '[ecc_converter id=' + this.state.post.id + ']';

			this.props.className += ' has-post';
		} else {

			this.props.className += ' no-post';
		}
		return [
			!! this.props.isSelected && (
					<SelectControl onChange={this.onChangeSelectPost} value={ this.props.attributes.selectedPost } label={ __( 'Select a Post' ) } options={ options } />
			),
			<div className={this.props.className}>{output}</div>
		]
	}
}


registerBlockType( 'cgb/block-ecc-block', {
	// Block name. Block names must be string that contains a namespace prefix. Example: my-plugin/my-custom-block.
	title: __( 'Easy Currency Converter' ), // Block title.
	icon: 'controls-repeat', // Block icon from Dashicons → https://developer.wordpress.org/resource/dashicons/.
	category: 'common', // Block category — Group blocks together based on common traits E.g. common, formatting, layout widgets, embed.
	keywords: [
		__( 'Easy Currency Converter' ),
		__( 'Converter' ),
		__( 'Currenty Converter' ),
	],

	attributes: {
		content: {
			type: 'array',
			source: 'children',
			selector: 'p',
		},
		title: {
			type: 'string',
			selector: 'h2'
		},
		link: {
			type: 'string',
			selector: 'a'
		},
		selectedPost: {
			type: 'number',
			default: 0,
		},
	},

	/**
	 * The edit function describes the structure of your block in the context of the editor.
	 * This represents what the editor will render when the block is used.
	 *
	 * The "edit" property must be a valid function.
	 *
	 * @link https://wordpress.org/gutenberg/handbook/block-api/block-edit-save/
	 */

	edit: mySelectPosts,

	/**
	 * The save function defines the way in which the different attributes should be combined
	 * into the final markup, which is then serialized by Gutenberg into post_content.
	 *
	 * The "save" property must be specified and must be a valid function.
	 *
	 * @link https://wordpress.org/gutenberg/handbook/block-api/block-edit-save/
	 */
	save: function( props ) {
		return (
			'[ecc_converter id='+props.attributes.id+']'
		)
	},
} );
